From ec244388c85a5cccc0277b5d50a672b6c6b575de Mon Sep 17 00:00:00 2001
From: Julien 'Lta' BALLET <contact@lta.io>
Date: Sat, 11 May 2013 13:14:56 +0200
Subject: [PATCH] qsv.c: Fixes real-time streaming dts issue, handle overflow

---
 modules/codec/qsv.c | 17 +++++++----------
 1 file changed, 7 insertions(+), 10 deletions(-)

diff --git a/modules/codec/qsv.c b/modules/codec/qsv.c
index 8303f3d..c767237 100644
--- a/modules/codec/qsv.c
+++ b/modules/codec/qsv.c
@@ -274,7 +274,6 @@ struct encoder_sys_t
     uint64_t         async_depth;         // Number of parallel encoding operations.
     uint64_t         first_task;          // The next sync point to be synchronized.
     async_task_t     *tasks;              // The async encoding tasks.
-    mtime_t          last_dts;            // The next sync point to be synchronized.
     mtime_t          offset_pts;          // The pts of the first frame, to avoid conversion overflow.
 };
 
@@ -660,22 +659,20 @@ static block_t *Encode(encoder_t *this, picture_t *pic)
 
             if (task->bs.TimeStamp) {
                 block->i_pts = qsv_timestamp_to_mtime(task->bs.TimeStamp) + sys->offset_pts;
-                if (task->bs.DecodeTimeStamp)
+                if (task->bs.DecodeTimeStamp && task->bs.DecodeTimeStamp <= task->bs.TimeStamp)
                     block->i_dts = qsv_timestamp_to_mtime(task->bs.DecodeTimeStamp) + sys->offset_pts;
                 else {
                     /* HW encoder (sometimes) doesn't set the DecodeTimeStamp field
                        so we give decoder some (100ms) headroom. */
-                    msg_Dbg(enc, "Encode returning empty DTS");
-                    block->i_dts = block->i_pts - 100 * 1000;
+                    msg_Dbg(enc, "Encode returning empty DTS or DTS > PTS");
+                    if (block->i_pts <= 100 * 1000)
+                        block->i_dts = 0;
+                    else
+                        block->i_dts = block->i_pts - 100 * 1000;
                 }
             }
 
-            /* Checking dts is increasing monotonically */
-            if (block->i_dts <= sys->last_dts) {
-                msg_Warn(enc, "Encoder outputs a packet with non increasing DTS");
-                block->i_dts = sys->last_dts + 1000;
-            }
-            sys->last_dts = block->i_dts;
+            // msg_Dbg(enc, "block->i_pts = %lld, block->i_dts = %lld", block->i_pts, block->i_dts);
 
             if (task->bs.FrameType & MFX_FRAMETYPE_IDR || (task->bs.FrameType & MFX_FRAMETYPE_REF))
                 block->i_flags = BLOCK_FLAG_TYPE_I;
-- 
1.8.1.2

